from .classification import Classification
from .detection import Detection
from .obb_detection import ObbDetection
from .segment import Segment
from .face_detection import FaceDetection
from .face_embedding import FaceEmbedding
from .face import Face
from .person_detection import PersonDetection
